package net.laparola.ui.android;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import com.actionbarsherlock.app.SherlockFragmentActivity;
import com.actionbarsherlock.view.Menu;
import com.actionbarsherlock.view.MenuItem;
import com.actionbarsherlock.view.Window;

import net.laparola.R;
import net.laparola.core.VersioneInformazioni;
import net.laparola.core.Testi.TestoTipi;
import net.laparola.ui.LaParolaBrowser;
import net.laparola.ui.LaParolaBrowser.LaParolaBrowserStaticClient;
import net.laparola.ui.LaParolaSegnalibri.Segnalibro;
import net.laparola.ui.LaParolaUrl;
import net.laparola.ui.android.LaParolaFragment.MyRunnable;
import net.laparola.ui.android.actionbar.LibraryActionItemManager;
import net.laparola.ui.android.actionbar.ReferenceActionItemManager;
import net.laparola.ui.android.actionbar.SearchActionItemManager;
import net.laparola.ui.android.actionbar.TTSActionItemManager;
import net.laparola.ui.android.dialogs.MessageDialog;
import net.laparola.ui.android.dialogs.PanelsDialog;
import net.laparola.ui.android.dialogs.PopupDialog;
import net.laparola.ui.android.dialogs.StarredDialog;
import net.laparola.ui.android.dialogs.WebViewDialog;
import android.content.DialogInterface;
import android.content.DialogInterface.OnDismissListener;
import android.content.ActivityNotFoundException;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.support.v4.app.FragmentTransaction;
import android.view.*;
import android.widget.Toast;

public class LaParolaActivity extends SherlockFragmentActivity implements LaParolaBrowserStaticClient {
	public static final String LAPAROLA_PREFERENCES = "net.laparola";
	public static final int MAX_PANELS = 4;

	public static boolean apriLink(Context context, String link) {
		Uri laparolauri = Uri.parse(link);
		Intent intent = new Intent(Intent.ACTION_VIEW, laparolauri);
		intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
		try {
			context.startActivity(intent);
			return true;
		} catch (ActivityNotFoundException e) {
			return false;
		}
	}
	
	private LaParolaFragment activeFragment;
	
	private SearchActionItemManager searchActionItemManager;
	private ReferenceActionItemManager referenceActionItemManager;
	private LibraryActionItemManager libraryActionItemManager;
	private TTSActionItemManager ttsActionItemManager;

	private MenuItem forwardActionItem;
	private MenuItem nightModeActionItem;
	private MenuItem starActionItem;

	private boolean firstReferenceClicked = true;

	public SharedPreferences sharedPreferences;
	private boolean mGoingBack;
	private long lastBackPressedTime = 0;
	private boolean useVolumeKeys;
	private boolean mFinishedLoading = false;
	private boolean mNightMode;
	
	protected boolean isPaused;
	private FourPanesLayout fourPanesLayout;

	private List<LaParolaFragment> fragments;

	private LaParolaActivityInitUtility initUtility;

	private int mLastOrientation;

	private LaParolaUrl mSelectingPanelForOpeningUrl;
	private LaParolaFragment mSwitchingPanels;
	
	@Override
	public InputStream apriFile(String filename) {
		try {
			if ((new File(filename)).exists())
				return new FileInputStream(filename);
			return getAssets().open(filename);
		} catch (Exception e) {
			e.printStackTrace();
		}

		return null;
	}

	public boolean collapseActionViewsExcept(MenuItem exclude) {
		searchActionItemManager.collapse(exclude);
		referenceActionItemManager.collapse(exclude);
		libraryActionItemManager.collapse(exclude);
		ttsActionItemManager.collapse(exclude);

		return (!initUtility.isWorking());
	}

	@Override
	public boolean dispatchKeyEvent(KeyEvent event) {
		int keyCode = event.getKeyCode();
		if (event.getAction() == KeyEvent.ACTION_UP) {
			if (keyCode == KeyEvent.KEYCODE_BACK) {
				long milliTime = System.nanoTime() / 1000000;
				if (!mGoingBack && milliTime > lastBackPressedTime + 500) {
					// ignora due "indietro" troppo vicini
					lastBackPressedTime = milliTime;
					if (activeFragment != null && activeFragment.precedenteEsiste()) {
						mGoingBack = true;
						activeFragment.vaiAPrecendente();
						forwardActionItem.setEnabled(true);
						return true;
					}
					// lo lascia gestire al S.O.
				}
			} else if (keyCode == KeyEvent.KEYCODE_SEARCH) {
				if (searchActionItemManager.isExpanded()) {
					searchActionItemManager.expandActionView();
				} else {
					searchActionItemManager.search();
				}
				return true;
			} else if (useVolumeKeys && keyCode == KeyEvent.KEYCODE_VOLUME_DOWN && !ttsActionItemManager.isExpanded()) {
				if (activeFragment != null && activeFragment.pageDown(false) == false)
					activeFragment.goToNextUrl();
				return true;
			} else if (useVolumeKeys && keyCode == KeyEvent.KEYCODE_VOLUME_UP && !ttsActionItemManager.isExpanded()) {
				if (activeFragment != null && activeFragment.pageUp(false) == false)
					activeFragment.goToPreviousUrl();
				return true;
			}
    	} else if (useVolumeKeys && 
    			(keyCode == KeyEvent.KEYCODE_VOLUME_DOWN ||
    			 keyCode == KeyEvent.KEYCODE_VOLUME_UP) && !ttsActionItemManager.isExpanded()) {
			return true;
		}
		return super.dispatchKeyEvent(event);
	}
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);
		super.onCreate(savedInstanceState);

		sharedPreferences = getSharedPreferences(LAPAROLA_PREFERENCES, MODE_PRIVATE);

		setContentView(R.layout.main_activity);
		fourPanesLayout = (FourPanesLayout)findViewById(R.id.four_panes_layout);
		
		fragments = new ArrayList<LaParolaFragment>();		
		
		/*
		 * TODO : usare risorse mLaParolaBrowser.Stringhe.Errore_Nessuna_versione = getContext().getString(R.string.no_version_present);
		 * mLaParolaBrowser.Stringhe.Errore_Non_presente = getContext().getString(R.string.not_present);
		 */
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getSupportMenuInflater().inflate(R.menu.action_bar, menu);

		MenuItem searchActionItem = menu.findItem(R.id.menu_item_seach);
		MenuItem referenceActionItem = menu.findItem(R.id.menu_item_reference);
		MenuItem libraryActionItem = menu.findItem(R.id.menu_item_library);
		MenuItem ttsActionItem = menu.findItem(R.id.menu_item_tts);

		searchActionItemManager = new SearchActionItemManager(this, searchActionItem);
		referenceActionItemManager = new ReferenceActionItemManager(this, referenceActionItem);
		libraryActionItemManager = new LibraryActionItemManager(this, libraryActionItem);
		ttsActionItemManager = new TTSActionItemManager(this, ttsActionItem);
		
		forwardActionItem = menu.findItem(R.id.menu_item_forward);
		starActionItem = menu.findItem(R.id.menu_item_star);
		nightModeActionItem = menu.findItem(R.id.menu_item_night_mode);

		nightModeActionItem.setTitle(mNightMode ? R.string.night_mode_off : R.string.night_mode_on);

		if (Build.VERSION.SDK_INT <= 10) {
			menu.findItem(R.id.menu_item_copy).setVisible(true);
			menu.findItem(R.id.menu_item_share).setVisible(true);
		}

		// if (!getPackageManager().hasSystemFeature(PackageManager.FEATURE_TOUCHSCREEN_MULTITOUCH)) {
		menu.findItem(R.id.menu_item_zoom_in).setVisible(true);
		menu.findItem(R.id.menu_item_zoom_out).setVisible(true);
		// }

		onFinishedLoadingActivity();

		return super.onCreateOptionsMenu(menu);
	}

	@Override
	public void onDestroy() {
		LaParolaBrowser.chiudi();
		ttsActionItemManager.destroy();
		super.onDestroy();
	}

	private void updateStar() {
		LaParolaUrl currentUrl = activeFragment.getUrlCorrente();

		starActionItem.setEnabled(true);
		starActionItem.setIcon(R.drawable.ic_action_unstarred);

		if (currentUrl == null) {
			return;
		}

		if (currentUrl.schema.equals("lppreferiti") || currentUrl.schema.equals("lpcronologia")) {
			starActionItem.setEnabled(false);
		}

		Segnalibro s = LaParolaBrowser.cercaUrlTraPreferiti(currentUrl);
		if (s != null) {
			starActionItem.setIcon(R.drawable.ic_action_starred);
		}
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		if (initUtility.isWorking())
			return true;

		int itemId = item.getItemId();

		if (itemId == R.id.menu_item_forward) {
			activeFragment.vaiASuccessivo();
			return true;
		} else if (itemId == R.id.menu_item_bookmarks) {
			activeFragment.vaiAdUrl("lppreferiti:");
			return true;
		} else if (itemId == R.id.menu_item_history) {
			activeFragment.vaiAdUrl("lpcronologia:");
			return true;
		} else if (itemId == R.id.menu_item_copy) {
			copy(R.string.select_text_copy);
			return true;
		} else if (itemId == R.id.menu_item_star) {
			showStarDialog();
			return true;
		} else if (itemId == R.id.menu_item_settings) {
			startActivity(new Intent(this, LaParolaPreferencesActivity.class));
			return true;
		} else if (itemId == R.id.menu_item_zoom_in) {
			activeFragment.setTextZoom(activeFragment.getTextZoom() + 10);
			return true;
		} else if (itemId == R.id.menu_item_night_mode) {
			setNightMode(!mNightMode);
			return true;
		} else if (itemId == R.id.menu_item_zoom_out) {
			activeFragment.setTextZoom(activeFragment.getTextZoom() - 10);
			return true;
		} else if (itemId == R.id.menu_item_share) {
			share();
			return true;
		} else if (itemId == android.R.id.home) {
			activeFragment.vaiAHome();
			return true;
		}

		return super.onOptionsItemSelected(item);
	}

	private void copy(int id) {
		Toast.makeText(this, id, Toast.LENGTH_LONG).show();
		activeFragment.selectAndCopyText();
	}

	@SuppressWarnings("deprecation")
	private void share() {
		final android.text.ClipboardManager clipboard = (android.text.ClipboardManager) getSystemService(CLIPBOARD_SERVICE);

		final String t = "--";
		clipboard.setText(t);

		copy(R.string.select_text_share);

		Runnable r = new Runnable() {
			@Override
			public void run() {
				CharSequence text = t;
				while (text.equals(t)) {
					text = clipboard.getText();
					try {
						Thread.sleep(1000);
						if (isPaused) {
							return;
						}
					} catch (InterruptedException e) {
						//
					}
				}

				if (text != null) {
					Intent intent = new Intent(Intent.ACTION_SEND);
					intent.setType("text/plain");
					intent.putExtra(Intent.EXTRA_SUBJECT, R.string.share_subject);
					intent.putExtra(Intent.EXTRA_TEXT, text);
					startActivity(Intent.createChooser(intent, getString(R.string.share_with)));
				}
			}
		};

		new Thread(r).start();
	}

	private void showStarDialog() {
		LaParolaUrl urlCorrente = activeFragment.getUrlCorrente();
		if (urlCorrente == null)
			return;

		Segnalibro s = LaParolaBrowser.cercaUrlTraPreferiti(urlCorrente);

		StarredDialog bookmark = new StarredDialog(this);
		bookmark.show();
		bookmark.setDescription(s == null ? urlCorrente.getDescrizione() : s.nome);
		bookmark.url = urlCorrente;
		bookmark.setOnDismissListener(new OnDismissListener() {
			@Override
			public void onDismiss(DialogInterface dialog) {
				updateStar();
			}
		});
	}

	private void setNightMode(boolean b) {
		mNightMode = b;
		if (nightModeActionItem != null)
			nightModeActionItem.setTitle(b ? R.string.night_mode_off : R.string.night_mode_on);
		
		for (LaParolaFragment f : fragments)
			f.setNightMode(b);
	}

	@Override
	protected void onPause() {
		super.onPause();

		isPaused = true;

		savePreferences();

		LaParolaBrowser.setLaParolaBrowserStaticClient(null);
	}

	private void savePreferences() {
		SharedPreferences.Editor editor = sharedPreferences.edit();

		editor.putInt("fragmentsNumber", getPanesNumber());
		editor.putInt("fragmentsOrientation", fourPanesLayout.getOrientation() == FourPanesLayout.HORIZONTAL ? FourPanesLayout.VERTICAL : FourPanesLayout.HORIZONTAL);
		
		if (getActiveFragment() != null)
			editor.putString("lastVersion", getActiveFragment().getVersione());

		for (int i = 0; i < fragments.size(); i++) {
			String idx = i == 0 ? "" : String.valueOf(i);
			LaParolaFragment f = fragments.get(i);
			
			editor.putString("lastVersion" + idx, f.getVersione());
			editor.putInt("textZoom" + idx, f.getTextZoom());
			editor.putInt("syncColor" + idx, f.getSyncColor());
			LaParolaUrl urlCorrente = f.getUrlCorrente();
			if (urlCorrente != null) {
				editor.putString("lastUrl" + idx, urlCorrente.getUrl());
			}
		}
		
		editor.putBoolean("nightMode", mNightMode);

		editor.commit();
	}

	@Override
	protected void onResume() {
		super.onResume();

		isPaused = false;

		LaParolaBrowser.setLaParolaBrowserStaticClient(this);
		
		if (hasFinishedLoading())
			onTestiCambiati();   // non riceve le informazioni quando l'attività è in pausa
		
		applyPreferences();
	}

	@Override
	public void onTestiCambiati() {
		for (LaParolaFragment f : fragments)
			f.onTestiCambiati();
		
		libraryActionItemManager.onTestiCambiati();
		
		onVersionChanged();
	}

	private void applyPreferences() {
		if (!hasFinishedLoading()) {
			fourPanesLayout.postDelayed(new Runnable() {
				@Override
				public void run() {
					applyPreferences();
				}
			}, 100);
			return;
		}		

		if (isPaused) {
			return;
		}
		
		if (fragments.size() == 0) {
			int fragmentsNumber = sharedPreferences.getInt("fragmentsNumber", 1);
			int fragmentsOrientation = sharedPreferences.getInt("fragmentsOrientation", FourPanesLayout.HORIZONTAL);
			if (sharedPreferences.getBoolean("homeAtStart", true) || fragmentsNumber < 1)
				fragmentsNumber = 1;
			
			addLaParolaFragment(true);
			setPanes(fragmentsNumber, fragmentsOrientation, true, null);
			setActiveFragment(fragments.get(0));
		}		
		
		LaParolaBrowser.setMostraParagrafi(sharedPreferences.getBoolean("paragraphOrVerses", true));
		LaParolaBrowser.setMostraTitoli(sharedPreferences.getBoolean("showTitles", true));
		LaParolaBrowser.setPosizioneRiferimento(Integer.parseInt(sharedPreferences.getString("referencePlacement", "1")));
		LaParolaBrowser.setTipoRiferimento(Integer.parseInt(sharedPreferences.getString("referenceType", "0")));
		LaParolaBrowser.setRiferimentoInApice(sharedPreferences.getBoolean("referenceSuperscript", true));

		useVolumeKeys = sharedPreferences.getBoolean("useVolumeKeys", true);
		fourPanesLayout.setKeepScreenOn(sharedPreferences.getBoolean("keepScreenOn", false));
		setNightMode(sharedPreferences.getBoolean("nightMode", false));

		for (LaParolaFragment f : fragments) {
			f.aggiornaPagina();
		}
	}

	@Override
	public void mostraAiutoRicerca() {
		WebViewDialog webViewDialog = new WebViewDialog(this, "file:///android_asset/Guida_ricerca.html");
		webViewDialog.setTitle(R.string.help);
		webViewDialog.show();
	}

	@Override
	public void mostraPulisciCronologia() {
		MessageDialog m = new MessageDialog(this, R.string.history, R.string.clear_history);
		m.setYesNo(R.string.clear, android.R.string.cancel, new Runnable() {
			@Override
			public void run() {
				LaParolaBrowser.pulisciCronologia();
				activeFragment.aggiornaPagina();   // la pagina della cronologia
			}
		}, null);
		m.show();
	}

	public boolean getNightMode() {
		return mNightMode;
	}

	public boolean hasFinishedLoading() {
		return mFinishedLoading;
	}

	public void onLoadingChanged() {
		boolean show = false;
		for (int i = 0; i < getPanesNumber(); i++) {
			LaParolaFragment f = fragments.get(i);
			if (f.isLoading()) {
				show = true;
				break;
			}
		}
		setSupportProgressBarIndeterminateVisibility(show);
	}

	public void updateActionBar() {
		mGoingBack = false;
		forwardActionItem.setEnabled(activeFragment.successivoEsiste());

		LaParolaUrl currentUrl = activeFragment.getUrlCorrente();
        
		if (currentUrl != null && currentUrl.schema.equals("laparola")) {
            String versione = activeFragment.getVersione();
            libraryActionItemManager.setVersion(versione);
            referenceActionItemManager.updateBooks();
            
			if (currentUrl.lcv != null) {
				int b = currentUrl.lcv[0];
				int c = currentUrl.lcv[1];
				int v = currentUrl.lcv[2];

				referenceActionItemManager.select(b, c, v);
				if (firstReferenceClicked) {
					referenceActionItemManager.expand();
					firstReferenceClicked = false;
				}
			} else {
				referenceActionItemManager.select(0, 0, 0);
			}

			if (currentUrl.ricerca != null) {
				searchActionItemManager.select(currentUrl.ricerca);
			}

			if (currentUrl.brani != null) {
				referenceActionItemManager.select(currentUrl.brani);
			}
		} else {
			referenceActionItemManager.select(0, 0, 0);
		}

		updateStar();

		if (initUtility.isWorking() || activeFragment.inHome()) {
			getSupportActionBar().setDisplayHomeAsUpEnabled(false);
		} else {
			getSupportActionBar().setDisplayHomeAsUpEnabled(true);
		}
	}

	public void onVersionChanged() {
		if (referenceActionItemManager != null) {
			referenceActionItemManager.onVersionChanged();
			libraryActionItemManager.onVersionChanged();
		}
	}

	private void onFinishedLoadingActivity() {
		if (hasFinishedLoading())
			return;
		
		if (initUtility == null) {
			initUtility = new LaParolaActivityInitUtility(this);
			fourPanesLayout.post(new Runnable() {
				@Override
				public void run() {
					initUtility.init();
				}
			});
		}

		if (initUtility.isWorking()) {
			fourPanesLayout.postDelayed(new Runnable() {
				@Override
				public void run() {
					onFinishedLoadingActivity();
				}
			}, 100);
			return;
		}

		setFinishedLoading(true);
	}
	
	public void setActiveFragment (LaParolaFragment value) {
		if (activeFragment == value)
			return;
		
		activeFragment = value;
		fourPanesLayout.setSelectedPane(fragments.indexOf(value));
		
		if (activeFragment.isCreated()) {
			updateActionBar();
			
			LaParolaUrl url = activeFragment.getUrlCorrente();
			if (!getUltimaBibbiaSalvata().equals(url.versione)) {
				VersioneInformazioni informazioniTesto = LaParolaBrowser.getInformazioniTesto(url.versione);
				if (informazioniTesto != null && informazioniTesto.getTipo().contains(TestoTipi.BIBBIA)) {
					setUltimaBibbiaSalvata(url.versione);
				}
			}
		}
	}
	
	public LaParolaFragment getActiveFragment() {
		if (activeFragment == null && initUtility != null && !initUtility.isWorking()) {
			applyPreferences();   // lo crea
		}
		return activeFragment;
	}

	public void addLaParolaFragment(boolean restoring) {
		final int position = fragments.size();
		
		if (position == MAX_PANELS) {
			return;
		}
		
		LaParolaFragment fragment = new LaParolaFragment();
		String idx = position == 0 ? "" : String.valueOf(position);
		
		final String version = sharedPreferences.getString("lastVersion" + idx, "");
		
		final String goToUrl;
		final int zoom;
		
		if (restoring) {
			fragment.setSyncColor(sharedPreferences.getInt("syncColor" + idx, 0));
			
			zoom = sharedPreferences.getInt("textZoom" + idx, 100);
			if (position == 0) {
				if (sharedPreferences.getBoolean("homeAtStart", true)) {
					goToUrl = null;
				} else {
					goToUrl = sharedPreferences.getString("lastUrl" + idx, null);
				}
			} else {
				// String defUrl = getActiveFragment().inHome() ? null : getActiveFragment().getUrlCorrente().getUrl();
				String defUrl = null; //getActiveFragment().getUrlCorrente().getUrl();
				goToUrl = sharedPreferences.getString("lastUrl" + idx, defUrl);
			}
		} else {
			// lastUrl = getActiveFragment().inHome() ? null : getActiveFragment().getUrlCorrente().getUrl();
			goToUrl = getActiveFragment().getUrlCorrente().getUrl(); 
			zoom = getActiveFragment().getTextZoom();
		}
		
		final String ultimaBibbia = LaParolaBrowser.getUltimaBibbia();
		
		fragment.onCreateGoToUrl = goToUrl;
		fragment.onCreateViewRunnable = new MyRunnable() {
			@Override
			public void run(LaParolaFragment self) {
				self.setTextZoom(zoom, false);
				self.setVersione(version);
				if (self.onCreateGoToUrl != null) {
					self.vaiAdUrl(self.onCreateGoToUrl);
				} else if (position == 0) {
					self.setVersione(ultimaBibbia);
					self.vaiAHome();
				}
				
				/*
				if (myRunnable != null)
					myRunnable.run(self);
				}
				*/
			}
		};
		
		fragments.add(fragment);
		FragmentTransaction transaction = getSupportFragmentManager().beginTransaction();
		transaction.add(fourPanesLayout.getFrameId(position), fragment, String.valueOf(position));
		transaction.commit();
	}

	public void onZoomChanged(LaParolaFragment fragment, int zoom) {
		for (LaParolaFragment f : fragments) {
			if (f.isCreated()) {
				f.setTextZoom(zoom, false);
			}
		}
	}

	public int getPanesNumber() {
		return fourPanesLayout.getNumberPanes();
	}
	
	public int getPanesOrientation() {
		return fourPanesLayout.getOrientation();
	}

	public void setPanes(int numberPanes, int orientation, boolean restoring, int[] synccolors) {
		while (fragments.size() < numberPanes) {
			addLaParolaFragment(restoring);
		}
		
		mLastOrientation = orientation;
		
		fourPanesLayout.setPanes(numberPanes, orientation);
		
		if (synccolors != null)
			for (int i = 0; i < fragments.size(); i++)
				fragments.get(i).setSyncColor(synccolors[i]);
		//setActiveFragment(fragment);
	}

	public void setFinishedLoading(boolean value) {
		mFinishedLoading = value;
		
		if (value == true) {
			setSupportProgressBarIndeterminateVisibility(false);
			findViewById(R.id.loading).setVisibility(View.GONE);
		}
	}

	public int getSyncColor(int i) {
		if (i < fragments.size()) {
			return fragments.get(i).getSyncColor();
		} else {
			String idx = i == 0 ? "" : String.valueOf(i);
			return sharedPreferences.getInt("syncColor" + idx, 0); 
		}
	}

	public void syncPanels(LaParolaFragment master) {
		LaParolaUrl urlCorrente = master.getUrlCorrente();
		for (int i = 0; i < fragments.size(); i++) {
			LaParolaFragment slave = fragments.get(i);
			if (slave != master && slave.getSyncColor() == master.getSyncColor()) {
				slave.setIgnoreNextUrlForSync(true);
				slave.vaiAdUrl(urlCorrente.getUrlConAltraVersione(slave.getVersione(), true));
			}
		}
	}

	public void openInPopupWindow(LaParolaUrl url) {
		PopupDialog pd = new PopupDialog(this);
		pd.setUrl(url);
		pd.show();
	}

	public void openInNewPanel(String url, String versione) {
		final int numberPanes = fourPanesLayout.getNumberPanes();
		if (numberPanes < MAX_PANELS) {
			setPanes(numberPanes + 1, mLastOrientation, false, null);
			
			LaParolaFragment f = fragments.get(numberPanes);
			f.setSyncColor(getFreeSyncColor());
			LaParolaUrl lpUrl = LaParolaBrowser.nuovoUrl(url, versione);
			if (!f.isCreated()) {
				f.onCreateGoToUrl = lpUrl.getUrl();
			} else {
				f.vaiAdUrl(lpUrl);
			}
		} else {
			Toast.makeText(this, R.string.error_open_new_panel, Toast.LENGTH_LONG).show();
		}
	}

	private int getFreeSyncColor() {
		for (int r = 0; r < MAX_PANELS; r++) {
			boolean ok = true;
			for (int i = 0; i < fragments.size(); i++) {
				if (fragments.get(i).getSyncColor() == r) {
					ok = false;
					break;
				}
			}
			if (ok) return r;
		}
		return -1;
	}

	public void closeActivePanel() {
		int panesNumber = getPanesNumber();
		LaParolaFragment f = getActiveFragment();
		
		setPanes(panesNumber - 1, mLastOrientation, false, null);
		
		for (int i = 0; i < panesNumber; i++) {
			if (fragments.get(i) == f) {
				for (int j = i; j < panesNumber - 1; j++) {
					switchPanels(fragments.get(j), fragments.get(j + 1));
				}
				return;
			}
		}
	}

	public void switchPanels() {
		if (getPanesNumber() == 1) 
			return;
					
		(Toast.makeText(this, R.string.select_panel, Toast.LENGTH_LONG)).show();
		mSwitchingPanels = getActiveFragment();
	}

	public void selectPanelForOpening(String url, String versione) {
		selectPanelForOpening(LaParolaBrowser.nuovoUrl(url, versione));
	}
	
	public void selectPanelForOpening(LaParolaUrl url) {
		if (getPanesNumber() == 1) {
			LaParolaFragment f = getActiveFragment();
			//String versione = f.getUrlCorrente().versione;
			//f.vaiAdUrl(url.getUrlConAltraVersione(versione));
			
			f.vaiAdUrl(url);
		} else {
			(Toast.makeText(this, R.string.select_panel, Toast.LENGTH_LONG)).show();
			mSelectingPanelForOpeningUrl = url;
		}
	}

	public void onFragmentTouch(LaParolaFragment mFragment) {
		setActiveFragment(mFragment);
		
		if (mSelectingPanelForOpeningUrl != null) {
			//String versione = mFragment.getUrlCorrente().versione;
			//mFragment.vaiAdUrl(mSelectingPanelForOpening.getUrlConAltraVersione(versione));
			mFragment.vaiAdUrl(mSelectingPanelForOpeningUrl);
			
			for (int i = 0; i < fragments.size(); i++) {
				LaParolaFragment f = fragments.get(i);
				if (f != mFragment && f.getSyncColor() == mFragment.getSyncColor()) {
					(Toast.makeText(this, R.string.link_broken, Toast.LENGTH_LONG)).show();
					mFragment.setSyncColor(getFreeSyncColor());
					break;
				}
			}
			
			mSelectingPanelForOpeningUrl = null;
		} else if (mSwitchingPanels != null) {
			switchPanels(mFragment, mSwitchingPanels);
			
			mSwitchingPanels = null;
		}
	}

	private void switchPanels(LaParolaFragment f1, LaParolaFragment f2) {
		int c1 = f1.getSyncColor();
		int c2 = f2.getSyncColor();
		LaParolaUrl u1 = f1.getUrlCorrente();
		LaParolaUrl u2 = f2.getUrlCorrente();
		
		f1.setSyncColor(c2);
		f2.setSyncColor(c1);
		
		f1.vaiAdUrl(u2);
		f2.vaiAdUrl(u1);
	}

	public void openInNewPanel(LaParolaUrl url) {
		openInNewPanel(url.getUrl(), url.versione);
	}

	public void openInPopupWindow(String url, String versione) {
		openInPopupWindow(LaParolaBrowser.nuovoUrl(url, versione));
	}

	@Override
	public String getUltimaBibbiaSalvata() {
		return sharedPreferences.getString("lastBible", "");
	}

	@Override
	public void setUltimaBibbiaSalvata(String versione) {
		SharedPreferences.Editor editor = sharedPreferences.edit();
		editor.putString("lastBible", versione);
		editor.commit();
	}

	public void showPanelsManagment() {
		PanelsDialog diag = new PanelsDialog(this);
		diag.show();
	}
}
