package net.laparola.ui.android;

import java.util.ArrayList;
import java.util.List;

import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager.NameNotFoundException;

import net.laparola.ui.LaParolaBrowser;
import net.laparola.ui.LaParolaSegnalibri;
import net.laparola.ui.LaParolaUrl;
import net.laparola.ui.android.bibleview.BibleView;

public class LaParolaJavascriptInterface {
	private BibleView mBibleView;

	public LaParolaJavascriptInterface(BibleView bibleView) {
		mBibleView = bibleView;
	}

	public String getSegnalibriCasuali(int n) {
		StringBuilder sb = new StringBuilder();
		List<LaParolaSegnalibri.GruppoSegnalibri> g = LaParolaBrowser.getGruppiSegnalibri();

		if (g.size() < n) return "";
		
		List<Integer> fatti = new ArrayList<Integer>();
		int s;
		for (int i = 0; i < n; i++) {
			do {
				s = (int) Math.floor(Math.random() * g.size());
			} while (fatti.contains(s));
			fatti.add(s);
			LaParolaSegnalibri.appendLinkGruppo(sb, g.get(s));
		}

		return sb.toString();
	}
	
	public String getVersettoCasuale () {
		LaParolaUrl url = mBibleView.getBrowser().getVersettoCasuale();
		return url.getUrl();
	}

	public String getVersioneProgramma() {
		PackageInfo packageInfo;
		try {
			Context context = mBibleView.getContext();
			packageInfo = context.getPackageManager().getPackageInfo(context.getPackageName(), 0);
			return packageInfo.versionName;
		} catch (NameNotFoundException e) {
			return "";
		}
	}
	
	public String normalizzaRiferimento (String riferimento, String versione) {
		return LaParolaBrowser.normalizzaRiferimento(riferimento, versione);
	}
	
	public String normalizzaRiferimento (String riferimento) {
		return LaParolaBrowser.normalizzaRiferimento(riferimento, getVersione());
	}
	
	public String convertiRiferimentoAStandard (String riferimento, String versione) {
		return LaParolaBrowser.convertiRiferimentoAStandard(riferimento, versione);
	}
	
	public String getVersione () {
		return mBibleView.getBrowser().getVersione();
	}
	
	public boolean getAggionamentiDisponibiliDebole () {
		// Restituisce se esistono aggiornamenti o meno, ma non blocca l'esecuzione:
		// se il file aggiorna.xml è in cache restituisce il risultato, altrimenti 
		// restituisce no e scarica il file in un thread separato. 
		
		LaParolaActivity laParolaActivity = (LaParolaActivity)mBibleView.getContext();
		String cacheFileName = laParolaActivity.sharedPreferences.getString("storagePath", "") + "/aggiorna.xml.cachedebole";
		
		return LaParolaBrowser.getAggionamentiDisponibiliDebole(cacheFileName);
	}
	
	public void notificaPrimoSegnalibroVisisible (String s) {
		mBibleView.onScrolledToAnchor(s);
	}
}
