package net.laparola.ui.android.dialogs;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import net.laparola.R;
import android.annotation.TargetApi;
import android.app.Dialog;
import android.content.Context;
import android.graphics.drawable.Drawable;
import android.os.Build;
import android.view.View;
import android.view.ViewGroup.LayoutParams;
import android.view.Window;
import android.widget.Button;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

public class HoloDialog extends Dialog implements android.view.View.OnClickListener {
	protected FrameLayout custom;
	protected LinearLayout topPanel;
	protected TextView message;
	protected ImageView icon;
	protected LinearLayout buttonPanel;
	protected Button button1;
	protected Button button2;
	protected Button button3;
	protected TextView alertTitle;
	protected Context mContext;
	protected LinearLayout contentPanel;
	protected View divider1;
	protected View divider2;
	private Runnable mOnYes;
	private Runnable mOnNo;

	public HoloDialog(Context context, boolean isAlert) {
		super(context, R.style.Theme_IgnazioHolo_Dialog);
		mContext = context;

		if (Build.VERSION.SDK_INT < 11 || isAlert) {
			getWindow().requestFeature(Window.FEATURE_NO_TITLE);

			super.setContentView(R.layout.holo_alert_dialog);
			topPanel = (LinearLayout) findViewById(R.id.topPanel);
			icon = (ImageView) findViewById(R.id.icon);
			alertTitle = (TextView) findViewById(R.id.alertTitle);
			contentPanel = (LinearLayout) findViewById(R.id.contentPanel);
			message = (TextView) findViewById(R.id.message);
			custom = (FrameLayout) findViewById(R.id.custom);
			buttonPanel = (LinearLayout) findViewById(R.id.buttonPanel);
			button2 = (Button) findViewById(R.id.button2);
			divider1 = findViewById(R.id.divider1);
			button3 = (Button) findViewById(R.id.button3);
			divider2 = findViewById(R.id.divider2);
			button1 = (Button) findViewById(R.id.button1);

			message.setVisibility(View.GONE);
			custom.setVisibility(View.GONE);

			buttonPanel.setVisibility(View.GONE);
			button2.setVisibility(View.GONE);
			divider1.setVisibility(View.GONE);
			button3.setVisibility(View.GONE);
			divider2.setVisibility(View.GONE);
			button1.setVisibility(View.GONE);

			setCloseOnTouchOutside(true);
		}
	}

	protected void setCloseOnTouchOutside(boolean value) {
		Method m;
		try {
			m = Window.class.getMethod("setCloseOnTouchOutside", boolean.class);
			m.invoke(getWindow(), value);
		} catch (NoSuchMethodException e) {
			//
		} catch (IllegalArgumentException e) {
			//
		} catch (IllegalAccessException e) {
			//
		} catch (InvocationTargetException e) {
			//
		}
	}

	@Override
	public void setContentView(View view) {
		LinearLayout.LayoutParams layoutParams = new LinearLayout.LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.WRAP_CONTENT);
		super.setContentView(view, layoutParams);
	}

	@Override
	public void setContentView(int layoutResID) {
		if (custom == null) {
			super.setContentView(layoutResID);
		} else {
			custom.setVisibility(View.VISIBLE);
			custom.removeAllViews();
			getLayoutInflater().inflate(layoutResID, custom);
		}
	}

	@Override
	public void setContentView(View view, LayoutParams params) {
		if (custom == null) {
			super.setContentView(view, params);
		} else {
			custom.removeAllViews();
			if (view != null) {
				custom.setVisibility(View.VISIBLE);
				custom.addView(view, params);
			} else {
				custom.setVisibility(View.GONE);
			}
		}
	}

	@Override
	public void setTitle(int titleId) {
		if (alertTitle == null) {
			super.setTitle(titleId);
		} else {
			alertTitle.setText(titleId);
		}
	}

	@Override
	public void setTitle(CharSequence title) {
		if (alertTitle == null) {
			super.setTitle(title);
		} else {
			alertTitle.setText(title);
		}
	}

	public void setIcon(Drawable d) {
		icon.setImageDrawable(d);
	}

	public void setIcon(int i) {
		icon.setImageResource(i);
	}

	public void setYesNo(int yesStringId, int noStringId, Runnable onYes, Runnable onNo) {
		buttonPanel.setVisibility(View.VISIBLE);
		button1.setOnClickListener(this);
		button2.setOnClickListener(this);

		button1.setText(yesStringId);
		button1.setVisibility(View.VISIBLE);

		if (noStringId != 0) {
			button2.setText(noStringId);

			button2.setVisibility(View.VISIBLE);
			divider1.setVisibility(View.VISIBLE);
		} else {
			button2.setVisibility(View.GONE);
			divider1.setVisibility(View.GONE);
		}

		mOnYes = onYes;
		mOnNo = onNo;
	}

	public void setButtons(int string1Id, int string2Id, int string3Id) {
		buttonPanel.setVisibility(View.VISIBLE);
		button1.setOnClickListener(this);
		button2.setOnClickListener(this);
		button3.setOnClickListener(this);

		button1.setText(string1Id);
		button1.setVisibility(View.VISIBLE);

		button2.setText(string2Id);
		button2.setVisibility(View.VISIBLE);
		divider1.setVisibility(View.VISIBLE);

		if (string3Id != 0) {
			button3.setText(string3Id);
			button3.setVisibility(View.VISIBLE);
			divider2.setVisibility(View.VISIBLE);
		}
	}

	@Override
	public void onClick(View v) {
		if (v == button2 && mOnNo != null) {
			mOnNo.run();
		} else if (v == button1 && mOnYes != null) {
			mOnYes.run();
		}

		dismiss();
	}

	@TargetApi(11)
	protected void setSoftwareRendererV11() {
		contentPanel.setLayerType(View.LAYER_TYPE_SOFTWARE, null);
	}
}
